@@ -31,25 +31,31 @@ class Level0Schema(Schema):
     """
 
     def __init__(self, instrument=None):
+        allowed_keys = set(self.sections.keys())
         if instrument:
             if instrument not in KNOWN_INSTRUMENT_TABLES.keys():
                 raise ValueError(
                     f"{instrument} does not match one of the known "
                     "instrument table names: {tuple(KNOWN_INSTRUMENT_TABLES.keys())}"
                 )
-            for table in KNOWN_INSTRUMENT_TABLES.keys():
-                if table == instrument:
-                    continue
-                self.sections.pop(table)
+            drop_tables = set(KNOWN_INSTRUMENT_TABLES.keys())
+            drop_tables.discard(instrument)
+            allowed_keys = allowed_keys.difference(drop_tables)
 
-        super().__init__(self.sections_from_dicts(self.sections.values()))
+        super().__init__(
+            self.sections_from_dicts(
+                (s for name, s in self.sections.items() if name in allowed_keys)
+            )
+        )
 
 
 class Spec122Schema(Level0Schema):
     @cached_property
     def sections(self) -> Dict:
-        print()
-        return spec122.load_spec122()
+        sections = spec122.load_spec122()
+        return {
+            key: section for key, section in sections.items() if key != "compression"
+        }
 
 
 class Level0Spec214Schema(Level0Schema):
