@@ -6,7 +6,10 @@ from dkist_data_simulator.spec214.vbi import (
     MosaicedVBIBlueDataset,
     TimeDependentVBIDataset,
 )
-from dkist_data_simulator.spec214.visp import TimeDependentVISPDataset
+from dkist_data_simulator.spec214.visp import (
+    SimpleVISPDataset,
+    TimeDependentVISPDataset,
+)
 from dkist_data_simulator.spec214.vtf import SimpleVTFDataset
 
 
@@ -83,3 +86,21 @@ def test_vtf_stokes_time():
     headers = Table(ds.generate_headers())
     time = np.unique(headers["DATE-AVG"])
     assert time.shape == (4,)
+
+
+def test_visp_4d():
+    ds = SimpleVISPDataset(
+        n_steps=2, n_maps=1, n_stokes=4, time_delta=10, linewave=500 * u.nm
+    )
+
+    headers = Table(ds.generate_headers())
+    assert headers[0]["DTYPE4"] == "STOKES"
+    assert "DTYPE5" not in headers.colnames
+
+    ds = SimpleVISPDataset(
+        n_steps=2, n_maps=2, n_stokes=1, time_delta=10, linewave=500 * u.nm
+    )
+
+    headers = Table(ds.generate_headers())
+    assert headers[0]["DTYPE4"] == "TEMPORAL"
+    assert "DTYPE5" not in headers.colnames
