@@ -3,9 +3,9 @@
 import io
 from pathlib import Path
 
-from astropy.io import fits
 import numpy as np
 import pytest  # type: ignore
+from astropy.io import fits
 
 from dkist_data_simulator.dataset import Dataset, key_function
 from dkist_data_simulator.spec122 import Spec122Schema
@@ -14,12 +14,12 @@ from dkist_data_simulator.spec122 import Spec122Schema
 class DatasetTest(Dataset):
     @property
     def data(self):
-        return np.empty(self.array_shape)
+        return np.random.random(self.array_shape)
 
 
 @pytest.fixture
 def dataset_1d():
-    ds_shape = (1, 11, 10)
+    ds_shape = (2, 11, 10)
     array_shape = (1, 11, 10)
     return DatasetTest(Spec122Schema(), ds_shape, array_shape)
 
@@ -79,7 +79,6 @@ def test_index(dataset_5d):
 
 
 def test_dataset_subclass():
-
     class TestDataset(Dataset):
         @key_function("INSTRUME")
         def instrume(self, key):
@@ -92,7 +91,6 @@ def test_dataset_subclass():
 
 
 def test_dataset_subclass_tuple():
-
     class TestDataset(Dataset):
         @key_function("INSTRUME", "OBSERVAT")
         def instrume(self, key):
@@ -117,6 +115,7 @@ def test_generate_headers(dataset_5d):
     def make_index(ds, key):
         ind = int(key[-1])
         return ds.file_index[3 - ind]
+
     dataset_5d.add_generator_function(("DINDEX1", "DINDEX2", "DINDEX3"), make_index)
 
     with pytest.warns(UserWarning):
@@ -150,7 +149,9 @@ def test_generate_hdu_uncompressed(dataset_1d):
     for ds in dataset_1d:
         hdu = ds.hdu()
         assert isinstance(hdu, fits.PrimaryHDU)
-        assert hdu.header['INSTRUME'] == "CADAIR"
+        assert hdu.header["INSTRUME"] == "CADAIR"
+        target = io.BytesIO()
+        hdu.writeto(target, output_verify="exception", overwrite=True, checksum=True)
 
 
 def test_generate_hdu_compressed(dataset_1d):
@@ -159,7 +160,9 @@ def test_generate_hdu_compressed(dataset_1d):
     for ds in dataset_1d:
         hdu = ds.hdu(rice_compress=True)
         assert isinstance(hdu, fits.CompImageHDU)
-        assert hdu.header['INSTRUME'] == "CADAIR"
+        assert hdu.header["INSTRUME"] == "CADAIR"
+        target = io.BytesIO()
+        hdu.writeto(target, output_verify="exception", overwrite=True, checksum=True)
 
 
 def test_generate_hdu_compressed_kwargs(dataset_1d):
@@ -170,3 +173,14 @@ def test_generate_hdu_compressed_kwargs(dataset_1d):
         assert isinstance(hdu, fits.CompImageHDU)
         assert hdu.header["INSTRUME"] == "CADAIR"
         assert hdu._header["ZCMPTYPE"] == "GZIP_1"
+        target = io.BytesIO()
+        hdu.writeto(target, output_verify="exception", overwrite=True, checksum=True)
+
+
+@pytest.mark.parametrize("file_count", [1, 2])
+def test_iteration(file_count):
+    dataset_shape = (file_count, 10, 10)
+    array_shape = (1, 10, 10)
+    dataset = Dataset(Spec122Schema(), dataset_shape, array_shape)
+    counter = [1 for _ in dataset]
+    assert sum(counter) == file_count
